@@ -1,23 +1,17 @@
-import six
+from sys import maxsize
+
 from construct.lib.py3compat import int2byte
-from construct.lib import (BitStreamReader, BitStreamWriter, encode_bin, decode_bin)
-from construct.core import (Struct, MetaField, StaticField, FormatField,
-    OnDemand, Pointer, Switch, Value, RepeatUntil, MetaArray, Sequence, Range,
-    Select, Pass, SizeofError, Buffered, Restream, Reconfig)
-from construct.adapters import (BitIntegerAdapter, PaddingAdapter,
-    ConstAdapter, CStringAdapter, LengthValueAdapter, IndexingAdapter,
-    PaddedStringAdapter, FlagsAdapter, StringAdapter, MappingAdapter)
-try:
-    from sys import maxsize
-except ImportError:
-    from sys import maxint as maxsize
+from construct.lib import BitStreamReader, BitStreamWriter, encode_bin, decode_bin
+from construct.core import Construct, Struct, MetaField, StaticField, FormatField, OnDemand, Pointer, Switch, Computed, RepeatUntil, MetaArray, Sequence, Range, Select, Pass, SizeofError, ArrayError, StringError, Buffered, Restream, Reconfig, Padding, Const, Aligned
+from construct.core import _read_stream, _write_stream
+from construct.adapters import BitIntegerAdapter, CStringAdapter, IndexingAdapter, PaddedStringAdapter, FlagsAdapter, StringAdapter, MappingAdapter
 
 
 #===============================================================================
 # fields
 #===============================================================================
 def Field(name, length):
-    """
+    r"""
     A field consisting of a specified number of bytes.
 
     :param name: the name of the field
@@ -30,7 +24,7 @@ def Field(name, length):
     else:
         return StaticField(name, length)
 
-def BitField(name, length, swapped = False, signed = False, bytesize = 8):
+def BitField(name, length, swapped=False, signed=False, bytesize=8):
     r"""
     BitFields, as the name suggests, are fields that operate on raw, unaligned
     bits, and therefore must be enclosed in a BitStruct. Using them is very
@@ -67,7 +61,6 @@ def BitField(name, length, swapped = False, signed = False, bytesize = 8):
         >>> foo.parse("\xe1\x1f")
         Container(a = 7, b = False, bar = Container(d = 15, e = 1), c = 8)
     """
-
     return BitIntegerAdapter(Field(name, length),
         length,
         swapped=swapped,
@@ -75,22 +68,8 @@ def BitField(name, length, swapped = False, signed = False, bytesize = 8):
         bytesize=bytesize
     )
 
-def Padding(length, pattern = six.b("\x00"), strict = False):
-    r"""A padding field (value is discarded)
-
-    :param length: the length of the field. the length can be either an integer,
-                   or a function that takes the context as an argument and returns the length
-    :param pattern: the padding pattern (character) to use. default is "\x00"
-    :param strict: whether or not to raise an exception is the actual padding
-                   pattern mismatches the desired pattern. default is False.
-    """
-    return PaddingAdapter(Field(None, length),
-        pattern = pattern,
-        strict = strict,
-    )
-
-def Flag(name, truth = 1, falsehood = 0, default = False):
-    """
+def Flag(name, truth=1, falsehood=0, default=False):
+    r"""
     A flag.
 
     Flags are usually used to signify a Boolean value, and this construct
@@ -257,24 +236,43 @@ def Array(count, subcon):
         con._clear_flag(con.FLAG_DYNAMIC)
     return con
 
-def PrefixedArray(subcon, length_field = UBInt8("length")):
-    """An array prefixed by a length field.
 
+class PrefixedArray(Construct):
+    r"""
+    An array prefixed by a length field.
+
+    :param lengthfield: a field returning an integer
     :param subcon: the subcon to be repeated
-    :param length_field: a construct returning an integer
-    """
-    def _length(ctx):
-      if issubclass(ctx.__class__, (list, tuple)):
-        return len(ctx)
-      return ctx[length_field.name]
 
-    return LengthValueAdapter(
-        Sequence(subcon.name,
-            length_field,
-            Array(_length, subcon),
-            nested = False
-        )
-    )
+    Example::
+
+        PrefixedArray(UBInt8("array"), UBInt8(None))
+        .parse(b"\x03\x01\x01\x01") -> [1,1,1]
+        .build([1,1,1]) -> b"\x03\x01\x01\x01"
+    """
+
+    def __init__(self, subcon, lengthfield=UBInt8(None)):
+        if not isinstance(lengthfield, Construct):
+            raise TypeError("lengthfield should be a Construct field")
+        if not isinstance(subcon, Construct):
+            raise TypeError("subcon should be a Construct field")
+        super(PrefixedArray, self).__init__(subcon.name)
+        self.lengthfield = lengthfield
+        self.subcon = subcon
+    def _parse(self, stream, context):
+        try:
+            count = self.lengthfield._parse(stream, context)
+            return list(self.subcon._parse(stream, context) for i in range(count))
+        except Exception:
+            raise ArrayError("could not read prefix or enough elements, stream too short")
+    def _build(self, obj, stream, context):
+        self.lengthfield._build(len(obj), stream, context)
+        for element in obj:
+            self.subcon._build(element, stream, context)
+    def _sizeof(self, context):
+        # return self.lengthfield._sizeof(None) + sum(self.subcon._sizeof(element) for element in context)
+        raise SizeofError("cannot calculate size")
+
 
 def OpenRange(mincount, subcon):
     return Range(mincount, maxsize, subcon)
@@ -310,7 +308,7 @@ def GreedyRange(subcon):
 
 def OptionalGreedyRange(subcon):
     r"""
-    Repeats the given unit zero or more times. This repeater can't
+    Repeats the given subcon zero or more times. This repeater can't
     fail, as it accepts lists of any length.
 
     :param subcon: construct to repeat
@@ -335,14 +333,19 @@ def OptionalGreedyRange(subcon):
 # subconstructs
 #===============================================================================
 def Optional(subcon):
-    """An optional construct. if parsing fails, returns None.
+    r"""
+    An optional construct.
+
+    If parsing fails, returns None. If building fails, writes nothing.
 
     :param subcon: the subcon to optionally parse or build
     """
     return Select(subcon.name, subcon, Pass)
 
+
 def Bitwise(subcon):
-    """Converts the stream to bits, and passes the bitstream to subcon
+    r"""
+    Converts the stream to bits, and passes the bitstream to subcon.
 
     :param subcon: a bitwise construct (usually BitField)
     """
@@ -366,30 +369,11 @@ def Bitwise(subcon):
             resizer = resizer)
     return con
 
-def Aligned(subcon, modulus = 4, pattern = six.b("\x00")):
-    r"""Aligns subcon to modulus boundary using padding pattern
-
-    :param subcon: the subcon to align
-    :param modulus: the modulus boundary (default is 4)
-    :param pattern: the padding pattern (default is \x00)
-    """
-    if modulus < 2:
-        raise ValueError("modulus must be >= 2", modulus)
-    def padlength(ctx):
-        return (modulus - (subcon._sizeof(ctx) % modulus)) % modulus
-    return SeqOfOne(subcon.name,
-        subcon,
-        # ??????
-        # ??????
-        # ??????
-        # ??????
-        Padding(padlength, pattern = pattern),
-        nested = False,
-    )
 
 def SeqOfOne(name, *args, **kw):
-    r"""A sequence of one element. only the first element is meaningful, the
-    rest are discarded
+    r"""
+    A sequence of one element. Only the first element is meaningful, the
+    rest are discarded.
 
     :param name: the name of the sequence
     :param \*args: subconstructs
@@ -398,14 +382,16 @@ def SeqOfOne(name, *args, **kw):
     return IndexingAdapter(Sequence(name, *args, **kw), index = 0)
 
 def Embedded(subcon):
-    """Embeds a struct into the enclosing struct.
+    r"""
+    Embeds a struct into the enclosing struct, merging fields.
 
     :param subcon: the struct to embed
     """
     return Reconfig(subcon.name, subcon, subcon.FLAG_EMBED)
 
 def Rename(newname, subcon):
-    """Renames an existing construct
+    r"""
+    Renames an existing construct.
 
     :param newname: the new name
     :param subcon: the subcon to rename
@@ -413,19 +399,23 @@ def Rename(newname, subcon):
     return Reconfig(newname, subcon)
 
 def Alias(newname, oldname):
-    """Creates an alias for an existing element in a struct
+    r"""
+    Creates an alias for an existing element in a struct.
+
+    Does not build.
 
     :param newname: the new name
     :param oldname: the name of an existing element
     """
-    return Value(newname, lambda ctx: ctx[oldname])
+    return Computed(newname, lambda ctx: ctx[oldname])
 
 
 #===============================================================================
 # mapping
 #===============================================================================
-def SymmetricMapping(subcon, mapping, default = NotImplemented):
-    """Defines a symmetrical mapping: a->b, b->a.
+def SymmetricMapping(subcon, mapping, default=NotImplemented):
+    r"""
+    Defines a symmetrical mapping: a->b, b->a.
 
     :param subcon: the subcon to map
     :param mapping: the encoding mapping (a dict); the decoding mapping is
@@ -443,7 +433,8 @@ def SymmetricMapping(subcon, mapping, default = NotImplemented):
     )
 
 def Enum(subcon, **kw):
-    r"""A set of named values mapping.
+    r"""
+    A set of named values mapping.
 
     :param subcon: the subcon to map
     :param \*\*kw: keyword arguments which serve as the encoding mapping
@@ -455,7 +446,8 @@ def Enum(subcon, **kw):
     return SymmetricMapping(subcon, kw, kw.pop("_default_", NotImplemented))
 
 def FlagsEnum(subcon, **kw):
-    r"""A set of flag values mapping.
+    r"""
+    A set of flag values mapping.
 
     :param subcon: the subcon to map
     :param \*\*kw: keyword arguments which serve as the encoding mapping
@@ -467,7 +459,8 @@ def FlagsEnum(subcon, **kw):
 # structs
 #===============================================================================
 def AlignedStruct(name, *subcons, **kw):
-    r"""A struct of aligned fields
+    r"""
+    A struct of aligned fields
 
     :param name: the name of the struct
     :param \*subcons: the subcons that make up this structure
@@ -476,7 +469,8 @@ def AlignedStruct(name, *subcons, **kw):
     return Struct(name, *(Aligned(sc, **kw) for sc in subcons))
 
 def BitStruct(name, *subcons):
-    r"""A struct of bitwise fields
+    r"""
+    A struct of bitwise fields
 
     :param name: the name of the struct
     :param \*subcons: the subcons that make up this structure
@@ -484,136 +478,75 @@ def BitStruct(name, *subcons):
     return Bitwise(Struct(name, *subcons))
 
 def EmbeddedBitStruct(*subcons):
-    r"""An embedded BitStruct. no name is necessary.
+    r"""
+    An embedded BitStruct. no name is necessary.
 
     :param \*subcons: the subcons that make up this structure
     """
     return Bitwise(Embedded(Struct(None, *subcons)))
 
+
 #===============================================================================
 # strings
 #===============================================================================
-def String(name, length, encoding=None, padchar=None, paddir="right",
-        trimdir="right"):
-    r"""
-    A configurable, fixed-length string field.
 
-    The padding character must be specified for padding and trimming to work.
-
-    :param name: name
-    :param length: length, in bytes
-    :param encoding: encoding (e.g. "utf8") or None for no encoding
-    :param padchar: optional character to pad out strings
-    :param paddir: direction to pad out strings; one of "right", "left", or "both"
-    :param str trim: direction to trim strings; one of "right", "left"
-
-    Example::
-
-        >>> from construct import String
-        >>> String("foo", 5).parse("hello")
-        'hello'
-        >>>
-        >>> String("foo", 12, encoding = "utf8").parse("hello joh\xd4\x83n")
-        u'hello joh\u0503n'
-        >>>
-        >>> foo = String("foo", 10, padchar = "X", paddir = "right")
-        >>> foo.parse("helloXXXXX")
-        'hello'
-        >>> foo.build("hello")
-        'helloXXXXX'
-    """
-    con = StringAdapter(Field(name, length), encoding=encoding)
-    if padchar is not None:
-        con = PaddedStringAdapter(con, padchar=padchar, paddir=paddir,
-            trimdir=trimdir)
-    return con
-
-def PascalString(name, length_field=UBInt8("length"), encoding=None):
+class PascalString(Construct):
     r"""
     A length-prefixed string.
 
-    ``PascalString`` is named after the string types of Pascal, which are
-    length-prefixed. Lisp strings also follow this convention.
+    ``PascalString`` is named after the string types of Pascal, which are length-prefixed. Lisp strings also follow this convention.
 
-    The length field will appear in the same ``Container`` as the
-    ``PascalString``, with the given name.
+    The length field will not appear in the same ``Container``, when parsing. Only the string will be returned. When building, the length is taken from len(the string). The length field can be anonymous (name is None) and can be variable length (such as VarInt).
 
     :param name: name
-    :param length_field: a field which will store the length of the string
-    :param encoding: encoding (e.g. "utf8") or None for no encoding
+    :param lengthfield: a field which will store the length of the string
+    :param encoding: encoding (e.g. "utf8") or None for bytes
 
     Example::
 
-        >>> foo = PascalString("foo")
-        >>> foo.parse("\x05hello")
-        'hello'
-        >>> foo.build("hello world")
-        '\x0bhello world'
-        >>>
-        >>> foo = PascalString("foo", length_field = UBInt16("length"))
-        >>> foo.parse("\x00\x05hello")
-        'hello'
-        >>> foo.build("hello")
-        '\x00\x05hello'
+        PascalString("string", ULInt32(None))
+        .parse(b"\x05\x00\x00\x00hello") -> "hello"
+        .build("hello") -> -> b"\x05\x00\x00\x00hello"
+
+        PascalString("string", ULInt32(None), encoding="utf8")
+        .parse(b"\x05\x00\x00\x00hello") -> u"hello"
+        .build(u"hello") -> -> b"\x05\x00\x00\x00hello"
     """
-
-    return StringAdapter(
-        LengthValueAdapter(
-            Sequence(name,
-                length_field,
-                Field("data", lambda ctx: ctx[length_field.name]),
-            )
-        ),
-        encoding=encoding,
-    )
-
-def CString(name, terminators=six.b("\x00"), encoding=None,
-            char_field=Field(None, 1)):
-    r"""
-    A string ending in a terminator.
-
-    ``CString`` is similar to the strings of C, C++, and other related
-    programming languages.
-
-    By default, the terminator is the NULL byte (b``0x00``).
-
-    :param name: name
-    :param terminators: sequence of valid terminators, in order of preference
-    :param encoding: encoding (e.g. "utf8") or None for no encoding
-    :param char_field: construct representing a single character
-
-    Example::
-
-        >>> foo = CString("foo")
-        >>> foo.parse(b"hello\x00")
-        b'hello'
-        >>> foo.build(b"hello")
-        b'hello\x00'
-        >>> foo = CString("foo", terminators = b"XYZ")
-        >>> foo.parse(b"helloX")
-        b'hello'
-        >>> foo.parse(b"helloY")
-        b'hello'
-        >>> foo.parse(b"helloZ")
-        b'hello'
-        >>> foo.build(b"hello")
-        b'helloX'
-    """
-
-    return Rename(name,
-        CStringAdapter(
-            RepeatUntil(lambda obj, ctx: obj in terminators, char_field),
-            terminators=terminators,
-            encoding=encoding,
-        )
-    )
+    __slots__ = ["name", "lengthfield", "encoding"]
+    def __init__(self, name, lengthfield=UBInt8(None), encoding=None):
+        super(PascalString, self).__init__(name)
+        self.lengthfield = lengthfield
+        self.encoding = encoding
+    def _parse(self, stream, context):
+        length = self.lengthfield._parse(stream, context)
+        obj = _read_stream(stream, length)
+        if self.encoding:
+            if isinstance(self.encoding, str):
+                obj = obj.decode(self.encoding)
+            else:
+                obj = self.encoding.decode(obj)
+        return obj
+    def _build(self, obj, stream, context):
+        if self.encoding:
+            if isinstance(self.encoding, str):
+                obj = obj.encode(self.encoding)
+            else:
+                obj = self.encoding.encode(obj)
+        else:
+            if not isinstance(obj, bytes):
+                raise StringError("no encoding provided but building from unicode string?")
+        self.lengthfield._build(len(obj), stream, context)
+        _write_stream(stream, len(obj), obj)
+    def _sizeof(self, context):
+        raise SizeofError("cannot calculate size")
 
 
 #===============================================================================
 # conditional
 #===============================================================================
 def IfThenElse(name, predicate, then_subcon, else_subcon):
-    """An if-then-else conditional construct: if the predicate indicates True,
+    r"""
+    An if-then-else conditional construct: if the predicate indicates True,
     `then_subcon` will be used; otherwise `else_subcon`
 
     :param name: the name of the construct
@@ -628,8 +561,9 @@ def IfThenElse(name, predicate, then_subcon, else_subcon):
         }
     )
 
-def If(predicate, subcon, elsevalue = None):
-    """An if-then conditional construct: if the predicate indicates True,
+def If(predicate, subcon, elsevalue=None):
+    r"""
+    An if-then conditional construct: if the predicate indicates True,
     subcon will be used; otherwise, `elsevalue` will be returned instead.
 
     :param predicate: a function taking the context as an argument and returning True or False
@@ -640,15 +574,16 @@ def If(predicate, subcon, elsevalue = None):
     return IfThenElse(subcon.name,
         predicate,
         subcon,
-        Value("elsevalue", lambda ctx: elsevalue)
+        Computed("elsevalue", lambda ctx: elsevalue)
     )
 
 
 #===============================================================================
 # misc
 #===============================================================================
-def OnDemandPointer(offsetfunc, subcon, force_build = True):
-    """An on-demand pointer.
+def OnDemandPointer(offsetfunc, subcon, force_build=True):
+    r"""
+    An on-demand pointer.
 
     :param offsetfunc: a function taking the context as an argument and returning
                        the absolute stream position
@@ -660,17 +595,4 @@ def OnDemandPointer(offsetfunc, subcon, force_build = True):
         force_build = force_build
     )
 
-def Magic(data):
-    """A 'magic number' construct. it is used for file signatures, etc., to validate
-    that the given pattern exists.
-
-    Example::
-
-        elf_header = Struct("elf_header",
-            Magic("\x7fELF"),
-            # ...
-        )
-    """
-    return ConstAdapter(Field(None, len(data)), data)
-
 
